@@ -29,6 +29,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 // imports
+#![macro_use]
 use std::fmt;
 use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
@@ -37,8 +38,7 @@ use num::{Float, Num, Zero, One};
 use crate::vector2::*;
 use crate::traits::LinearAlgebra;
 use crate::slices_methods::*;
-
-// code
+use crate::utils::nearly_equal;
 
 /// A static Matrix of 2x2 shape
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -46,21 +46,22 @@ pub struct M22<T>([[T; 2]; 2]);
 
 impl<T> M22<T> {
 
-    pub fn new(data_input: [[T; 2]; 2]) -> Self {
+    pub const fn new(data_input: [[T; 2]; 2]) -> Self {
         Self(data_input)
     }
 
-    pub fn create(a: T, b: T, c: T, d: T) -> Self {
+    pub const fn create(a: T, b: T, c: T, d: T) -> Self {
         Self::new([[a, b], [c, d]])
     }
 
-    pub fn rows(&self) -> usize {
+    pub const fn rows(&self) -> usize {
         self.0.len()
     }
 
-    pub fn cols(&self) -> usize {
+    pub const fn cols(&self) -> usize {
         self.rows()
     }
+
 }
 
 impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
@@ -98,7 +99,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
         let c = self[(1, 0)];
         let d = self[(1, 1)];
         let det = self.det();
-        if det.abs() > T::epsilon() {
+        if !nearly_equal(det, T::zero(), T::epsilon()) {
             Some(M22::new([[d / det, -b / det], [-c / det, a / det]]))
         } else {
             None
@@ -108,8 +109,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
     /// Calculate de QR factorization of the M22 via gram-schmidt
     /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
-        let det = self.det();
-        if det.abs() > T::epsilon() {
+        if !nearly_equal(self.det(), T::zero(), T::epsilon()) {
             let cols = self.get_cols();
             let mut q: [V2<T>; 2] = *M22::zeros().get_cols();
             for i in 0..q.len() {
@@ -176,7 +176,6 @@ impl<T: Num + Copy> M22<T> {
         }
         result
     }
-
 }
 
 // M22 * V2
@@ -399,12 +398,14 @@ impl<T> From<[[T; 2]; 2]> for M22<T> {
 
 impl<T> Index<(usize, usize)> for M22<T> {
     type Output = T;
+    #[inline(always)]
     fn index(&self, index: (usize, usize)) -> &T {
         &self.0[index.0][index.1]
     }
 }
 
 impl<T> IndexMut<(usize, usize)> for M22<T> {
+    #[inline(always)]
     fn index_mut(&mut self, index: (usize, usize)) -> &mut T {
         &mut self.0[index.0][index.1]
     }
@@ -431,6 +432,12 @@ impl<T: Num + fmt::Display> fmt::Display for M22<T> {
     }
 }
 
+//-------------------------------------------------------------------------
+//                        constants
+//-------------------------------------------------------------------------
+// TODO(elsuizo): no se si me sirve esto pero lo podriamos dejar
+pub const M22_ZEROS: M22<f32> = M22::new([[0.0, 0.0], [0.0, 0.0]]);
+pub const M22_IDENT: M22<f32> = M22::new([[1.0, 0.0], [0.0, 1.0]]);
 //-------------------------------------------------------------------------
 //                        testing
 //-------------------------------------------------------------------------
@@ -517,15 +524,13 @@ mod test_matrix2x2 {
         assert_eq!(result.as_vec(), expected.as_vec());
     }
 
-    // TODO(elsuizo:2020-06-02): no se como hacer para que ande con Ints y Floats
-    // #[test]
-    // #[ignore]
-    // fn test_determinant() {
-    //     let m1 = M22::new([[1, 2], [3, 4]]);
-    //     let result = m1.det();
-    //     let expected = -2;
-    //     assert_eq!(result, expected);
-    // }
+    #[test]
+    fn test_determinant() {
+        let m1 = M22::new([[1.0, 2.0], [1.0, 2.0]]);
+        let result = m1.det();
+        let expected = 0.0;
+        assert_eq!(result, expected);
+    }
 
     #[test]
     fn product_with_vector2_rhs_test() {
@@ -570,6 +575,13 @@ mod test_matrix2x2 {
         }
     }
 
+    #[test]
+    fn inverse_fail() {
+        let m1 = M22::new([[1.0, 2.0], [1.0, 2.0]]);
+        let result = m1.inverse();
+        assert!(result.is_none())
+    }
+
     #[test]
     fn get_columns_test() {
         let m1 = m22_new!(1.0, 2.0;
